<?php

class FWMA_Settings {
	private $flywire_multicurrency_settings_options;
	private $settings_options_name;

	public function __construct() {
		add_action( 'admin_menu', array( $this, 'fwma_settings_add_plugin_page' ) );
		add_action( 'admin_init', array( $this, 'fwma_settings_page_init' ) );
		$this->settings_options_name = 'fwma_options';
	}

	public function fwma_settings_add_plugin_page() {
		add_options_page(
			'Flywire for Woocommerce Multicurrency Settings', // page_title
			'Flywire Multicurrency', // menu_title
			'manage_options', // capability
			'flywire-multicurrency-settings', // menu_slug
			array( $this, 'fwma_settings_create_admin_page' ) // function
		);
	}

	public function fwma_settings_create_admin_page() {
		$this->flywire_multicurrency_settings_options = get_option( 'fwma_options' ); ?>

        <div class="wrap">
            <h2>Flywire for Woocommerce Multicurrency Add-On Settings</h2>
            <p>Set up additional portals/currencies to be used with WooCommerce. Enter a
                comma-delimited list of demo and/or production portals, then the corresponding comma-delimited list of currencies that those
                portals support.  The number of portals and currencies must match.</p>

            <form method="post" action="options.php">
				<?php
				settings_fields( 'flywire_multicurrency_settings_option_group' );
				do_settings_sections( 'flywire-multicurrency-settings-admin' );
				submit_button();
				?>
            </form>
        </div>
	<?php }

	public function fwma_settings_page_init() {
		register_setting(
			'flywire_multicurrency_settings_option_group', // option_group
			$this->settings_options_name, // option_name
			array( $this, 'fwma_settings_sanitize' ) // sanitize_callback
		);

		add_settings_section(
			'flywire_multicurrency_settings_section', // id
			'Settings', // title
			array( $this, 'fwma_settings_section_info' ), // callback
			'flywire-multicurrency-settings-admin' // page
		);

		add_settings_field(
			'demo_portals', // id
			'Demo Portals', // title
			array( $this, 'fwma_demo_portals_callback' ), // callback
			'flywire-multicurrency-settings-admin', // page
			'flywire_multicurrency_settings_section' // section
		);

		add_settings_field(
			'prod_portals', // id
			'Production Portals', // title
			array( $this, 'fwma_prod_portals_callback' ), // callback
			'flywire-multicurrency-settings-admin', // page
			'flywire_multicurrency_settings_section' // section
		);

		add_settings_field(
			'currencies', // id
			'Currencies', // title
			array( $this, 'fwma_currencies_callback' ), // callback
			'flywire-multicurrency-settings-admin', // page
			'flywire_multicurrency_settings_section' // section
		);
	}

	public function fwma_settings_sanitize( $input ) {
		$valid           = true;
		$sanitary_values = array();

		// either demo or prod portals is required
		if ( '' == $input['demo_portals'] && '' == $input['prod_portals'] ) {
			add_settings_error(
				$this->settings_options_name,
				'portals',
				'You must supply at least one Demo or Production Portal.',
				'error'
			);
			$valid = false;
		}

		// portals must be 3-letter codes
		if ( '' !== $input['demo_portals'] && ! preg_match( '/^([a-zA-Z]{3}(,\s*)*)+$/', $input['demo_portals'] ) ) {
			add_settings_error(
				$this->settings_options_name,
				'demo_portals',
				__('Demo Portals must be a comma-delimited list of 3-letter portal codes (e.g. "ABC, XYZ, FOO").', 'flywire_multicurrency'),
				'error'
			);
			$valid = false;
		}

		// portals must be 3-letter codes
		if ( '' !== $input['prod_portals'] && ! preg_match( '/^([a-zA-Z]{3}(,\s*)*)+$/', $input['prod_portals'] ) ) {
			add_settings_error(
				$this->settings_options_name,
				'prod_portals',
				__('Production Portals must be a comma-delimited list of 3-letter portal codes (e.g. "ABC, XYZ, FOO").', 'flywire_multicurrency'),
				'error'
			);
			$valid = false;
		}

		// currencies is required
		if ( '' == $input['currencies'] ) {
			add_settings_error(
				$this->settings_options_name,
				'currencies',
				__('Currencies is a required field.', 'flywire_multicurrency'),
				'error'
			);
			$valid = false;
        // currencies must be 3-letter codes
		} else if ( ! preg_match( '/^([a-zA-Z]{3}(,\s*)*)+$/', $input['currencies'] ) ) {
			add_settings_error(
				$this->settings_options_name,
				'currencies',
				__('Currencies must be a comma-delimited list of 3-letter currency codes (e.g. "USD, GBP, EUR").', 'flywire_multicurrency'),
				'error'
			);
			$valid = false;
		}

		// all filled out fields must have the same number of values
		if ( $valid ) {
			if ( '' !== $input['demo_portals'] && sizeof( explode( ',', $input['demo_portals'] ) ) !== sizeof( explode( ',', $input['currencies'] ) ) ) {
				$valid = false;
			} else if ( '' !== $input['prod_portals'] && sizeof( explode( ',', $input['prod_portals'] ) ) !== sizeof( explode( ',', $input['currencies'] ) ) ) {
				$valid = false;
			}

			if ( ! $valid ) {
				add_settings_error(
					$this->settings_options_name,
					'currencies',
					__('You must provide an equal number of currencies, demo and/or production portals.', 'flywire_multicurrency'),
					'error'
				);
			}
		}

		$sanitary_values['demo_portals'] = sanitize_text_field( trim( $input['demo_portals'] ) );
		$sanitary_values['prod_portals'] = sanitize_text_field( trim( $input['prod_portals'] ) );
		$sanitary_values['currencies'] = sanitize_text_field( trim( $input['currencies'] ) );

		return $sanitary_values;
	}

	public function fwma_settings_section_info() {

	}

	public function fwma_demo_portals_callback() {
		printf(
			'<input class="regular-text" type="text" name="fwma_options[demo_portals]" id="demo_portals" value="%s">',
			isset( $this->flywire_multicurrency_settings_options['demo_portals'] ) ? esc_attr( $this->flywire_multicurrency_settings_options['demo_portals'] ) : ''
		);
	}

	public function fwma_prod_portals_callback() {
		printf(
			'<input class="regular-text" type="text" name="fwma_options[prod_portals]" id="prod_portals" value="%s">',
			isset( $this->flywire_multicurrency_settings_options['prod_portals'] ) ? esc_attr( $this->flywire_multicurrency_settings_options['prod_portals'] ) : ''
		);
	}

	public function fwma_currencies_callback() {
		printf(
			'<input class="regular-text" type="text" name="fwma_options[currencies]" id="currencies" value="%s">',
			isset( $this->flywire_multicurrency_settings_options['currencies'] ) ? esc_attr( $this->flywire_multicurrency_settings_options['currencies'] ) : ''
		);
	}
}

if ( is_admin() ) {
	$flywire_multicurrency_settings = new FWMA_Settings();
}
